@@ -17,6 +17,7 @@ import com.unciv.logic.map.tile.RoadStatus
 import com.unciv.logic.map.tile.Tile
 import com.unciv.models.ruleset.tile.Terrain
 import com.unciv.models.ruleset.tile.TileImprovement
+import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import com.unciv.utils.Log
@@ -284,7 +285,7 @@ class WorkerAutomation(
                 .filter {
                     (it.civilianUnit == null || it == currentTile)
                             && (it.owningCity == null || it.getOwner()==civInfo)
-                            && (tileCanBeImproved(unit, it) || it.isPillaged())
+                            && getPriority(it) > 1
                             && it.getTilesInDistance(2)  // don't work in range of enemy cities
                         .none { tile -> tile.isCityCenter() && tile.getCity()!!.civ.isAtWarWith(civInfo) }
                             && it.getTilesInDistance(3)  // don't work in range of enemy units
@@ -292,13 +293,10 @@ class WorkerAutomation(
                 }
                 .sortedByDescending { getPriority(it) }
 
-        // the tile needs to be actually reachable - more difficult than it seems,
-        // which is why we DON'T calculate this for every possible tile in the radius,
-        // but only for the tile that's about to be chosen.
-        val selectedTile = workableTiles.firstOrNull { unit.movement.canReach(it) }
+        // These are the expensive calculations (tileCanBeImproved, canReach), so we only apply these filters after everything else it done.
+        val selectedTile = workableTiles.firstOrNull { unit.movement.canReach(it) && (tileCanBeImproved(unit, it) || it.isPillaged()) }
 
         return if (selectedTile != null
-                && getPriority(selectedTile) > 1
                 && (!workableTiles.contains(currentTile)
                     || getPriority(selectedTile) > getPriority(currentTile)))
             selectedTile
@@ -370,9 +368,14 @@ class WorkerAutomation(
         }
         if (potentialTileImprovements.isEmpty()) return null
 
+        val cityUniqueCaches = HashMap<City, LocalUniqueCache>()
         fun getRankingWithImprovement(improvementName: String): Float {
             val improvement = ruleSet.tileImprovements[improvementName]!!
-            val stats = tile.stats.getImprovementStats(improvement, civInfo, tile.getCity())
+            val city = tile.getCity()
+            val cache =
+                    if (city == null) LocalUniqueCache(false)
+                    else cityUniqueCaches.getOrPut(city) { LocalUniqueCache() }
+            val stats = tile.stats.getImprovementStats(improvement, civInfo, tile.getCity(), cache)
             return Automation.rankStatsValue(stats, unit.civ)
         }
 
@@ -381,7 +384,7 @@ class WorkerAutomation(
             .filter { it.second > 0f }
             .maxByOrNull { it.second }?.first
 
-        val lastTerrain = tile.getLastTerrain()
+        val lastTerrain = tile.lastTerrain
 
         fun isUnbuildableAndRemovable(terrain: Terrain): Boolean = terrain.unbuildable
                 && ruleSet.tileImprovements.containsKey(Constants.remove + terrain.name)
@@ -483,7 +486,7 @@ class WorkerAutomation(
         val enemyCivs = civInfo.getKnownCivs()
             .filterNot { it == civInfo || it.cities.isEmpty() || !civInfo.getDiplomacyManager(it).canAttack() }
         // no potential enemies
-        if (enemyCivs.isEmpty()) return false
+        if (enemyCivs.none()) return false
 
         val threatMapping: (Civilization) -> Int = {
             // the war is already a good nudge to build forts
@@ -501,7 +504,7 @@ class WorkerAutomation(
             civInfo,
             it) <= threatMapping(it) }
         // no threat, let's not build fort
-        if (enemyCivsIsCloseEnough.isEmpty()) return false
+        if (enemyCivsIsCloseEnough.none()) return false
 
         // make list of enemy cities as sources of threat
         val enemyCities = mutableListOf<Tile>()
