@@ -109,7 +109,7 @@ class Ruleset {
     }
 
     private fun <T : INamed> createHashmap(items: Array<T>): LinkedHashMap<String, T> {
-        val hashMap = LinkedHashMap<String, T>()
+        val hashMap = LinkedHashMap<String, T>(items.size)
         for (item in items)
             hashMap[item.name] = item
         return hashMap
@@ -506,8 +506,13 @@ object RulesetCache : HashMap<String,Ruleset>() {
                 debug("Mod loaded successfully: %s", modRuleset.name)
                 if (Log.shouldLog()) {
                     val modLinksErrors = modRuleset.checkModLinks()
-                    if (modLinksErrors.any()) {
-                        debug("checkModLinks errors: %s", modLinksErrors.getErrorText())
+                    // For extension mods which use references to base ruleset objects, the parameter type
+                    // errors are irrelevant - the checker ran without a base ruleset
+                    val logFilter: (RulesetError) -> Boolean =
+                        if (modRuleset.modOptions.isBaseRuleset) { { it.errorSeverityToReport > RulesetErrorSeverity.WarningOptionsOnly } }
+                        else { { it.errorSeverityToReport > RulesetErrorSeverity.WarningOptionsOnly && !it.text.contains("does not fit parameter type") } }
+                    if (modLinksErrors.any(logFilter)) {
+                        debug("checkModLinks errors: %s", modLinksErrors.getErrorText(logFilter))
                     }
                 }
             } catch (ex: Exception) {
