@@ -1,6 +1,5 @@
 package com.unciv.models.stats
 
-import com.unciv.Constants
 import com.unciv.models.translations.tr
 import kotlin.reflect.KMutableProperty0
 
@@ -152,7 +151,7 @@ open class Stats(
 
     /** Since notifications are translated on the fly, when saving stats there we need to do so in English */
     fun toStringForNotifications() = this.joinToString {
-        (if (it.value > 0) "+" else "") + it.value.toInt() + " " + it.key.toString().tr(Constants.english)
+        (if (it.value > 0) "+" else "") + it.value.toInt() + " " + it.key.toString()
     }
 
     // For display in diplomacy window
