@@ -1,6 +1,8 @@
 package com.unciv.ui.components
 
+import com.badlogic.gdx.Gdx
 import com.badlogic.gdx.graphics.Color
+import com.badlogic.gdx.graphics.GL20
 import com.badlogic.gdx.graphics.Pixmap
 import com.badlogic.gdx.graphics.Texture
 import com.badlogic.gdx.graphics.g2d.BitmapFont
@@ -8,12 +10,16 @@ import com.badlogic.gdx.graphics.g2d.BitmapFont.BitmapFontData
 import com.badlogic.gdx.graphics.g2d.BitmapFont.Glyph
 import com.badlogic.gdx.graphics.g2d.GlyphLayout
 import com.badlogic.gdx.graphics.g2d.PixmapPacker
+import com.badlogic.gdx.graphics.g2d.SpriteBatch
 import com.badlogic.gdx.graphics.g2d.TextureRegion
+import com.badlogic.gdx.graphics.glutils.FrameBuffer
+import com.badlogic.gdx.scenes.scene2d.Actor
 import com.badlogic.gdx.utils.Array
 import com.badlogic.gdx.utils.Disposable
 import com.unciv.Constants
 import com.unciv.GUI
 import com.unciv.UncivGame
+import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.translations.tr
 import com.unciv.ui.images.ImageGetter
 
@@ -154,6 +160,13 @@ class NativeBitmapFontData(
             MayaCalendar.katun -> getPixmap(MayaCalendar.katunIcon)
             MayaCalendar.baktun -> getPixmap(MayaCalendar.baktunIcon)
             in MayaCalendar.digits -> getPixmap(MayaCalendar.digitIcon(ch))
+            in Fonts.charToRulesetImageActor ->
+                try {
+                    // This sometimes fails with a "Frame buffer couldn't be constructed: incomplete attachment" error, unclear why
+                    Fonts.getPixmapFromActor(Fonts.charToRulesetImageActor[ch]!!)
+                } catch (ex: Exception) {
+                    Pixmap(0,0, Pixmap.Format.RGBA8888) // Empty space
+                }
             else -> fontImplementation.getCharPixmap(ch)
         }
     }
@@ -191,6 +204,7 @@ object Fonts {
         val settings = GUI.getSettings()
         fontImplementation.setFontFamily(settings.fontFamilyData, settings.getFontSize())
         font = fontImplementation.getBitmapFont()
+        font.data.markupEnabled = true
     }
 
     /** Reduce the font list returned by platform-specific code to font families (plain variant if possible) */
@@ -231,6 +245,77 @@ object Fonts {
         return pixmap
     }
 
+    val rulesetObjectNameToChar =HashMap<String, Char>()
+    val charToRulesetImageActor = HashMap<Char, Actor>()
+    // See https://en.wikipedia.org/wiki/Private_Use_Areas - char encodings 57344 63743 are not assigned
+    var nextUnusedCharacterNumber = 57344
+    fun addRulesetImages(ruleset:Ruleset) {
+        rulesetObjectNameToChar.clear()
+        charToRulesetImageActor.clear()
+        nextUnusedCharacterNumber = 57344
+
+        fun addChar(objectName:String, objectActor: Actor){
+            val char = Char(nextUnusedCharacterNumber)
+            nextUnusedCharacterNumber++
+            rulesetObjectNameToChar[objectName] = char
+            charToRulesetImageActor[char] = objectActor
+        }
+
+        for (resourceName in ruleset.tileResources.keys)
+            addChar(resourceName, ImageGetter.getResourcePortrait(resourceName, ORIGINAL_FONT_SIZE))
+
+        for (buildingName in ruleset.buildings.keys)
+            addChar(buildingName, ImageGetter.getConstructionPortrait(buildingName, ORIGINAL_FONT_SIZE))
+
+        for (unitName in ruleset.units.keys)
+            addChar(unitName, ImageGetter.getConstructionPortrait(unitName, ORIGINAL_FONT_SIZE))
+
+        for (promotionName in ruleset.unitPromotions.keys)
+            addChar(promotionName, ImageGetter.getPromotionPortrait(promotionName, ORIGINAL_FONT_SIZE))
+
+        for (improvementName in ruleset.tileImprovements.keys)
+            addChar(improvementName, ImageGetter.getImprovementPortrait(improvementName, ORIGINAL_FONT_SIZE))
+
+        for (techName in ruleset.technologies.keys)
+            addChar(techName, ImageGetter.getTechIconPortrait(techName, ORIGINAL_FONT_SIZE))
+
+        for (nation in ruleset.nations.values)
+            addChar(nation.name, ImageGetter.getNationPortrait(nation, ORIGINAL_FONT_SIZE))
+    }
+
+    val frameBuffer by lazy { FrameBuffer(Pixmap.Format.RGBA8888, Gdx.graphics.width, Gdx.graphics.height, false) }
+    val spriteBatch by lazy { SpriteBatch() }
+
+    fun getPixmapFromActor(actor: Actor): Pixmap {
+
+        frameBuffer.begin()
+
+        Gdx.gl.glClearColor(0f,0f,0f,0f)
+        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT)
+
+        spriteBatch.begin()
+        actor.draw(spriteBatch, 1f)
+        spriteBatch.end()
+
+        val w = actor.width.toInt()
+        val h = actor.height.toInt()
+        val pixmap = Pixmap(w, h, Pixmap.Format.RGBA8888)
+        Gdx.gl.glReadPixels(0, 0, w, h, GL20.GL_RGBA, GL20.GL_UNSIGNED_BYTE, pixmap.pixels)
+        frameBuffer.end()
+
+
+        // Pixmap is now *upside down* so we need to flip it around the y axis
+        for (i in 0..w)
+            for (j in 0..h/2) {
+                val topPixel = pixmap.getPixel(i,j)
+                val bottomPixel = pixmap.getPixel(i, h-j)
+                pixmap.drawPixel(i,j,bottomPixel)
+                pixmap.drawPixel(i,h-j,topPixel)
+            }
+
+        return pixmap
+    }
+
     const val turn = '⏳'               // U+23F3 'hourglass'
     const val strength = '†'            // U+2020 'dagger'
     const val rangedStrength = '‡'      // U+2021 'double dagger'
