@@ -20,8 +20,8 @@ import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.json.json
 import com.unciv.logic.city.PerpetualConstruction
-import com.unciv.models.ruleset.nation.Nation
 import com.unciv.models.ruleset.Ruleset
+import com.unciv.models.ruleset.nation.Nation
 import com.unciv.models.skins.SkinCache
 import com.unciv.models.tilesets.TileSetCache
 import com.unciv.ui.components.*
@@ -74,6 +74,9 @@ object ImageGetter {
 
         TileSetCache.assembleTileSetConfigs(ruleset.mods)
         SkinCache.assembleSkinConfigs(ruleset.mods)
+
+        Fonts.resetFont()
+        Fonts.addRulesetImages(ruleset)
     }
 
     /** Loads all atlas/texture files from a folder, as controlled by an Atlases.json */
@@ -448,8 +451,16 @@ object ImageGetter {
 
     fun getAvailableSkins() = ninePatchDrawables.keys.asSequence().map { it.split("/")[1] }.distinct()
 
-    fun getAvailableTilesets() = textureRegionDrawables.keys.asSequence().filter { it.startsWith("TileSets") && !it.contains("/Units/") }
-            .map { it.split("/")[1] }.distinct()
+    /** Determines available TileSets from the currently loaded Texture paths.
+     *
+     *  Note [TileSetCache] will not necessarily load all of them, e.g. if a Mod fails
+     *  to provide a config json for a graphic with a Tileset path.
+     *
+     *  Intersect with [TileSetCache.getAvailableTilesets] for a more reliable answer
+     */
+    fun getAvailableTilesets() = textureRegionDrawables.keys.asSequence()
+        .filter { it.startsWith("TileSets") && !it.contains("/Units/") }
+        .map { it.split("/")[1] }.distinct()
 
     fun getAvailableUnitsets() = textureRegionDrawables.keys.asSequence().filter { it.contains("/Units/") }
         .map { it.split("/")[1] }.distinct()
